﻿--[[
Name: LibGuildBankComm-1.0
Revision: $Revision: 1 $
Author(s): Myrroddin (psvander@gmail.com)
Documentation: http://www.wowace.com/index.php/LibResComm-1.0
SVN:  svn://svn.wowace.com/wow/librescomm-1-0/mainline/trunk
Description: Saves guild bank contents and sends/receives updates over the addon channel
Dependencies: LibStub, CallbackHandler-1.0, ChatThrottleLib, AceSerializer-3.0
]]

local MAJOR_VERSION = "LibGuildBankComm-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 1 $"):match("%d+"))

local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

-- Older version loaded. Clean up
if lib.Stop then
	lib.Stop()
end

-- localize global APIs --------------------------------------------------------
local _G = getfenv(0)
local BagnonFrameguildbank = _G.BagnonFrameguildbank
local ChatThrottleLib = _G.ChatThrottleLib
local GuildBankFrame = _G.GuildBankFrame
local GetGuildBankItemInfo = _G.GetGuildBankItemInfo
local GetGuildBankItemLink = _G.GetGuildBankItemLink
local GetGuildBankMoney = _G.GetGuildBankMoney
local GetNumGuildBankTabs = _G.GetNumGuildBankTabs
local GetGuildBankTabInfo = _G.GetGuildBankTabInfo
local GetGuildInfo = _G.GetGuildInfo
local IsInGuild = _G.IsInGuild
local LibStub = _G.LibStub
local string = _G.string
local UnitName = _G.UnitName
local wipe = _G.table.wipe

--------------------------------------------------------------------------------
-- Start library
--
local libSerial = LibStub("AceSerializer-3.0");

lib.Stop = function()
	lib.frame:UnregisterAllEvents()
end
lib:Start()

lib.guildBank = lib.guildBank or {}

if not lib.frame then
  lib.frame = CreateFrame("Frame")
end

lib.frame:SetScript("OnEvent", function(this, event, ...)
	this[event](this, ...)
end)
lib.frame:UnregisterAllEvents()

if not lib.Callbacks then
	lib.Callbacks = LibStub("CallbackHandler-1.0"):New(lib)
end

local guilded = IsInGuild("player")
local guildName = GetGuildInfo("player")
local pName = UnitName("player")

local function commSend(contents, distribution, target)
	ChatThrottleLib:SendAddonMessage("BULK", "gBankComm", contents, distribution, target)
end

function lib:Start()
	lib.frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	lib.frame:RegisterEvent("CHAT_MSG_ADDON")
	lib.frame:RegisterEvent("PLAYER_GUILD_UPDATE")
	if guilded == 1 then
		lib.frame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_OPENED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_CLOSED")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_MONEY")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_TABS")
	end
end

function lib.frame:CHAT_MSG_ADDON(prefix, msg, distribution, sender)
    if prefix ~= "gBankComm" then return end
    if sender == pName then return end
    
    local success, cmd, bankData = libSerial:Deserialize(msg)
    if not success then return end
    if cmd == "REQUEST" then
        local commString = libSerial:Serialize("UPDATE", lib.guildBank)
        commSend("gBankComm", commString, "WHISPER", sender)
    elseif cmd == "UPDATE" then
        lib.guildBank = bankData
        lib.CallBacks:Fire("GuildBankComm_ContentsChanged", sender, lib.guildBank)
    end
end

function lib:PLAYER_ENTERING_WORLD()
	commSend("gBankComm", libSerial:Serialize("REQUEST"), "GUILD")
end

function lib:PLAYER_GUILD_UPDATE()
	if guilded ~= 1 then
		wipe(lib.guildBank)
		lib.frame:UnregisterEvent("GUILDBANKBAGSLOTS_CHANGED")		
		lib.frame:UnregisterEvent("GUILDBANKFRAME_CLOSED")
		lib.frame:UnregisterEvent("GUILDBANK_UPDATE_MONEY")
		lib.frame:UnregisterEvent("GUILDBANK_UPDATE_TABS")
	else
		lib.frame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_OPENED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_CLOSED")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_MONEY")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_TABS")
	end
	guildName = GetGuildInfo("player")
	guilded = IsInGuild("player")
end

function lib:GUILDBANK_UPDATE_MONEY()
	-- sometimes money gets updated when nobody is there, like guild repairs or the upcoming tithing system
	lib.guildBank[guildName].Money = GetGuildBankMoney()
end

function lib:GUILDBANKFRAME_OPENED()
	lib.guildBank[guildName].numTabs = GetNumGuildBankTabs()
	lib.guildBank[guildName].Money = GetGuildBankMoney()
	
	local name, icon, view	
	for page = 1, lib.guildBank[guildName].numTabs do
		name, icon, view = GetGuildBankTabInfo(page)
		lib.guildBank[guildName].tab.texture[page] = icon
		lib.guildBank[guildName].tab.name[page] = name
		lib.guildBank[guildName].tab.view[page] = view
	end
end

function lib:GUILDBANKFRAME_CLOSED()
	local commString = libSerial:Serialize("UPDATE", lib.guildBank)
	commSend(string.format("gBankComm", commString), "GUILD")
	lib.CallBacks:Fire("GuildBankComm_ContentsChanged", unit, lib.guildBank)
end

function lib:GUILDBANK_UPDATE_TABS()
	lib.guildBank[guildName].numTabs = GetNumGuildBankTabs()
end

function lib:GUILDBANKBAGSLOTS_CHANGED()
	if (GuildBankFrame and GuildBankFrame:IsVisible()) or (BagnonFrameguildbank and BagnonFrameguildbank:IsVisible()) then
		local page = GetCurrentGuildBankTab()
		--98 slots on a page
		for slot = 1, 98 do
			--see if the slot has an item in it
			if GetGuildBankItemLink(page, slot) then
				--if there is link, parse the ID and place that in table w/ key = slot
				_, _, itemString = string.find(GetGuildBankItemLink(page, slot), "^|c%x+|H(.+)|h%[.+%]")
				lib.guildBank[guildName].links[page][slot] = itemString
				
				--get item info too, need for stack sizes in image
				_, num = GetGuildBankItemInfo(page, slot)
				lib.guildBank[guildName].info[page][slot] = num
			else
				--if there is no link, slot is empty... fill slot w/ empty string
				--needed to keep consistent tables with keys from 1 to 98
				lib.guildBank[guildName].links[page][slot] = "empty"
				lib.guildBank[guildName].info[page][slot] = "empty"
			end
			num, itemString = nil, nil
		end
	end
end